@@ -348,6 +348,9 @@ pub(crate) fn parse_keyring_internal<R>(mut input: R,
         t!("Found a {:?} at offset {}, length: {:?}",
            t, start_of_packet, header.length());
         match t {
+            // Ignore padding.
+            Padding => (),
+
             // Start of a new certificate.
             PublicKey | SecretKey => {
                 if start_of_packet > 0 {
@@ -389,14 +392,22 @@ pub(crate) fn parse_keyring_internal<R>(mut input: R,
             | SED
             | Literal
             | SEIP
-            | MDC
-            | AED =>
+            | MDC =>
             {
                 t!("Encountered a ({:?}) at offset {}, \
                     which does not belong in a certificate",
                    t, start_of_packet);
                 break;
-            }
+            },
+
+            t => if t.is_critical() {
+                t!("Encountered a ({:?}) at offset {}, \
+                    which does not belong in a certificate",
+                   t, start_of_packet);
+                break;
+            } else {
+                // Ignore unknown non-critical packet.
+            },
         }
 
         // Advance to the next packet.
@@ -552,14 +563,14 @@ mod tests {
         for i in 0..n {
             let (cert, _) =
                 CertBuilder::general_purpose(
-                    None, Some(format!("{}@example.org", i)))
+                    Some(format!("{}@example.org", i)))
                 .generate()?;
 
             cert.as_tsk().serialize(&mut data)?;
             certs_orig.push(cert);
 
             if literal {
-                let mut lit = Literal::new(DataFormat::Text);
+                let mut lit = Literal::new(DataFormat::Unicode);
                 lit.set_body(b"data".to_vec());
 
                 Packet::from(lit).serialize(&mut data)?;
@@ -775,14 +786,14 @@ mod tests {
 
         let (cert_1, _) =
             CertBuilder::general_purpose(
-                None, Some("a@example.org"))
+                Some("a@example.org"))
             .generate()?;
         let cert_1_packets: Vec<Packet>
             = cert_1.into_packets().collect();
 
         let (cert_2, _) =
             CertBuilder::general_purpose(
-                None, Some("b@example.org"))
+                Some("b@example.org"))
             .generate()?;
 
         for n in 1..cert_1_packets.len() {
